
#include "font.h"

#ifdef USE_PANGO
Bool
font_load (Wm                   *w, 
	   char                 *spec, 
	   MBPangoFont          *font)
#elif USE_XFT
Bool
font_load (Wm                   *w, 
	   char                 *spec, 
           XftFont              **font)
#else
Bool
font_load (Wm                   *w, 
	   char                 *spec, 
           XFontStruct          **font)
#endif
{
   const char delim[] = "|";
   char *orig = NULL, *str = NULL, *token = NULL;

   orig = str = strdup(spec);
      
   while( (token = strsep (&str, delim)) != NULL )
     {
#ifdef USE_PANGO
       int size = 8;
       char *nptr = spec, *sptr = spec, *dptr = NULL;

       while (*sptr != '-' && *sptr != '\0') sptr++;

       if (*sptr == '-')
	 {
	   *sptr = '\0';
	   sptr++;
	   dptr = sptr;
	   
	   while (*dptr != ':' && *dptr != '\0') dptr++;
	   
	   if (*dptr == ':') *dptr = '\0';
	   size = atoi(sptr);
	   if (size < 1) size = 8;
	 }
 
       font->fontdes = pango_font_description_new ();
       
       pango_font_description_set_family (font->fontdes, nptr);
       pango_font_description_set_size (font->fontdes, size * PANGO_SCALE); 
       
       font->font = pango_font_map_load_font (w->pgo_fontmap, 
					     w->pgo, 
					     font->fontdes);
       font->metrics = pango_font_get_metrics(font->font, NULL);

       font->ascent = PANGO_PIXELS(pango_font_metrics_get_ascent(font->metrics));
       font->descent = PANGO_PIXELS(pango_font_metrics_get_descent(font->metrics));

       if (orig) free(orig);
       return True;
#elif USE_XFT
       if ((*font = XftFontOpenName(w->dpy, w->screen, token)) != NULL)
	 { 
	   if (orig) free(orig); 
	   return True; 
	 }
#else
       if ((*font = XLoadQueryFont(w->dpy, token)) != NULL)
	 { 
	   if (orig) free(orig);  
	   return True; 
	 }
#endif
     }
      
   if (orig) free(orig);
   fprintf(stderr, "Matchbox: *warning* unable to load font : %s\n", spec);
      
   return False;
}


#ifdef USE_PANGO
void
font_paint (Wm                   *w, 
	    XftDraw              *drawable,
	    XftColor             *color,
	    MBPangoFont          *font,
	    PangoLayout          *pgo_layout,
	    unsigned char        *text,
	    int                   text_len,
	    int                   x,
	    int                   y)
#elif USE_XFT
void
font_paint (Wm                   *w, 
	    XftDraw              *drawable,
	    XftColor             *color,
	    XftFont              *font,
	    unsigned char        *text,
	    int                   text_len,
	    Bool                  text_is_utf8,
	    int                   x,
	    int                   y)
#else
void
font_paint (Wm                   *w, 
	    Drawable              drawable,
	    XColor               *color,
	    XFontStruct          *font,
	    GC                    gc,
	    unsigned char        *text,
	    int                   text_len,
	    int                   x,
	    int                   y)
#endif
{
#if defined(USE_PANGO)
      PangoLayoutLine *this = pango_layout_get_line(pgo_layout, 0);
      GSList *run;
      PangoRectangle logical_rect;
      PangoRectangle ink_rect;

      int x_off = 0;

      for (run = this->runs; run; run = run->next)
	{
	  PangoLayoutRun *this = run->data;
	  PangoRectangle rect;

	  dbg("%s() PANGO render run, rendering %s, xoff: %i\n", 
	      __func__, text, x_off );

	  pango_layout_index_to_pos (pgo_layout, this->item->offset, &rect);

	  pango_xft_render (drawable,
			    color,
			    font->font,
			    this->glyphs,
			    x + x_off, y);

	  pango_glyph_string_extents (this->glyphs, font->font, 
				      &ink_rect, &logical_rect);

	  x_off += PANGO_PIXELS(logical_rect.width);

	}
      g_object_unref (pgo_layout);
#elif defined(USE_XFT)
      if (text_is_utf8)
	XftDrawStringUtf8(drawable,
			  color, 
			  font,
			  x,
			  y,
			  text, 
			  ewmh_utf8_get_byte_cnt(text, text_len));
      else XftDrawString8(drawable,
			  color, 
			  font,
			  x,
			  y,
			  text, 
			  text_len);
#else
      XSetFont(w->dpy, gc, font->fid);
      XSetForeground(w->dpy, gc, color->pixel);
      XDrawString(w->dpy, drawable, gc, x, y, text, text_len);
#endif
}


#ifdef USE_PANGO
int
font_get_str_width (Wm                   *w, 
		    MBPangoFont          *font,
		    unsigned char        *text,
		    int                   text_len,
		    Bool                  text_is_utf8 )
#elif USE_XFT
int
font_get_str_width (Wm                   *w, 
		    XftFont              *font,
		    unsigned char        *text,
		    int                   text_len,
		    Bool                  text_is_utf8)
#else
int
font_get_str_width (Wm                   *w, 
		    XFontStruct          *font,
		    unsigned char        *text,
		    int                   text_len,
		    Bool                  text_is_utf8)
#endif
{
#ifdef USE_PANGO
  return (PANGO_PIXELS(pango_font_metrics_get_approximate_char_width (font->metrics))
	  * text_len); 		/* XXX this is kludgy */

#elif USE_XFT   
  XGlyphInfo extents;
  if (text_is_utf8)
     XftTextExtentsUtf8(w->dpy, font, (unsigned char *)text,
			ewmh_utf8_get_byte_cnt(text, text_len), 
			&extents);
   else
     XftTextExtents8(w->dpy, font, (unsigned char *)text, text_len, &extents);
   return extents.width;
#else
   return XTextWidth(font, text, text_len);
#endif
}
